@@ -31,12 +31,12 @@ const MessageT	cmd_MemoryDebug		= 1001;
 const MessageT	cmd_SetJapaneseFont	= 1100;
 const MessageT	cmd_SetRomanFont		= 1101;
 
-const UInt32	thTicksYield				= 6;
-const UInt32	thTicksForceSwitch			= 10;
-const UInt32	thTicksHeavyAlert			= 20;
-const UInt32	thTicksPriorityWeight		= 60;
+const UInt32	thTicksYield				= 2;  //  33 ms
+const UInt32	thTicksForceSwitch			= 6;  // 100 ms
+const UInt32	thTicksHeavyAlert			= 15; // 250 ms
+const UInt32	thTicksPriorityWeight		= 60; //   1 sec
 const UInt32	thDefaultPriority			= 4;
-const UInt32	thMillsecDeadLock			= 30000;
+const UInt32	thMillsecDeadLock			= 30000;//30 sec
 
 //#define THREAD_DEBUG_MSG
 
