@@ -33,7 +33,7 @@
 #define BACKGROUND_COLOR	COL(0x3ff,0x3ff,0x3ff)
 #define BACKGROUND_COLOR_MID	COL(0x200,0x200,0x200)
 
-
+int motion_cnt;
 
 extern GBVIEW_ENV env;
 
@@ -1305,15 +1305,12 @@ set_opac_mode();
 void
 opac_move()
 {
-
-set_cpu_msg(100);
+ss_printf("0P_MOVE\n");
 	del_tick(set_opac_mode);
-set_cpu_msg(101);
 	lock_task(opac_lock);
-set_cpu_msg(102);
 	_opac_move();
-set_cpu_msg(103);
 	unlock_task(opac_lock,"opac_move");
+ss_printf("0P_MOVE end\n");
 }
 
 
@@ -1364,8 +1361,6 @@ _opac_stop()
 int seq;
 void * mp;
 
-
-printf("opac_stop %x %x %i\n",cflame.ms,cflame.start_shot,cflame.ix.type);
 	if ( cflame.ms == 0 )
 		return;
 	if ( cflame.start_shot == 0 )
@@ -1389,10 +1384,13 @@ printf("opac_stop %x %x %i\n",cflame.ms,cflame.start_shot,cflame.ix.type);
 void
 opac_stop()
 {
+ss_printf("0P_STOP\n");
+
 	del_tick(set_opac_mode);
 	lock_task(opac_lock);
 	_opac_stop();
 	unlock_task(opac_lock,"opac_stop");
+ss_printf("0P_STOP END\n");
 }
 
 void
@@ -1413,10 +1411,12 @@ int size;
 void
 set_opac_mode(MOVE_SET * ms)
 {
+ss_printf("SET_0P_MODE\n");
 	del_tick(set_opac_mode);
 	lock_task(opac_lock);
 	_set_opac_mode(ms);
 	unlock_task(opac_lock,"set_opac_mode");
+ss_printf("SET_OP_MODE END\n");
 }
 
 int
@@ -2116,6 +2116,8 @@ set_cpu_msg(1);
 	case VES_BUTTON_PRESS:
 set_cpu_msg(2);
 
+printf("============================ PRESS\n");
+
 		cflame.mode = GBM_PRESS_IMAGE;
 		cflame.press_x = x;
 		cflame.press_y = y;
@@ -2156,7 +2158,7 @@ set_cpu_msg(3);
 		set_opac_mode(0);
 		break;
 	case VES_BUTTON_MOTION:
-set_cpu_msg(4);
+ss_printf("MOTION %i - %i %i\n",motion_cnt ++,cflame.mode,cflame.select_mode);
 
 		if ( cflame.mode == GBM_PRESS_IMAGE &&
 				cflame.select_mode == MODE_ROTATE ) {
@@ -2172,7 +2174,6 @@ set_cpu_msg(4);
 			cflame.press_x = x;
 			cflame.press_y = y;
 			opac_move();
-set_cpu_msg(5);
 
 			checkin_seq(-1);
 			v_redraw_image(cflame.im);
@@ -2180,12 +2181,14 @@ set_cpu_msg(5);
 		}
 		else if ( cflame.mode == GBM_PRESS_IMAGE &&
 				cflame.select_mode == MODE_GRAB ) {
+ss_printf("MOTION GRAB-1 %i\n",motion_cnt ++);
 			if ( check_event(cflame.im->header.win,
 				VE_BUTTON,VES_BUTTON_MOTION) )
 				return;
-set_cpu_msg(6);
 
 
+ss_printf("MOTION GRAB-2 %i\n",motion_cnt ++);
+
 			cflame.press_x = x;
 			cflame.press_y = y;
 			opac_move();
@@ -2195,7 +2198,6 @@ set_cpu_msg(6);
 			checkout_seq(-1);
 
 		}
-set_cpu_msg(7);
 		if ( cflame.mode == GBM_IDLE ) {
 			if ( check_event(cflame.im->header.win,
 				VE_BUTTON,VES_BUTTON_MOTION) )
@@ -2208,7 +2210,6 @@ set_cpu_msg(7);
 		if ( check_event(cflame.im->header.win,
 			VE_BUTTON,VES_BUTTON_MOTION) )
 			return;
-set_cpu_msg(8);
 		break;
 	default:
 		return;
@@ -2386,6 +2387,9 @@ VPOINT p;
 int i;
 static XL_INTERPRETER * xli;
 
+if ( subtype == VES_BUTTON_PRESS )
+printf("========================*****");
+printf("BTN SUBTYPE %i - %i\n",subtype,VES_BUTTON_PRESS);
 	if ( xli == 0 ) {
 		xli = new_xl_interpreter();
 		xli->a_type = XLA_SELF;
@@ -2677,6 +2681,9 @@ check_task()
 	}
 }
 
+
+
+
 void
 event_task(TKEY key)
 {
@@ -2689,9 +2696,6 @@ set_cpu_thr_type(21);
 	change_pri(0,PRI_USER_INTERFACE);
 	d = gb_window();
 	create_task(pixelmap_task,0,PRI_REDRAW);
-/*
-create_task(check_task,0,PRI_REDRAW);
-*/
 	v_loop(d);
 }
 
