@@ -64,6 +64,18 @@ XLISP_ENV * map_top,* map_cmd;
 	set_gv_resource(RT_MAP,map_top);
 }
 
+
+void
+free_clip(CLIP * c)
+{
+CLIP * cc;
+	for ( ; c ; ) {
+		cc = c;
+		c = c->next;
+		d_f_ree(cc);
+	}
+}
+
 void
 free_map_parameter(MAP_PARAMETER * mp)
 {
