@@ -26,16 +26,18 @@
 #include	"task.h"
 #include	"lock_level.h"
 #include	"pri_level.h"
-
+#include	"rs_cache.h"
 #define HANDLING_MAX		20
 
+
 extern SEM res_lock;
 extern GV_FUNC gv_new_table[RT_MAX];
 extern XLISP_ENV * gv_resource_env[RT_MAX];
 
-SYS_QUEUE res_q1,res_q2,res_lp1,res_disposer,res_extend;
+SYS_QUEUE res_q0,res_q1,res_q2,res_lp1,res_disposer,res_extend;
 LOAD_RESOURCE_WORK * first_list;
 
+void _load_resource_task0(TKEY d);
 void _load_resource_task1(TKEY d);
 void _load_resource_task2(TKEY d);
 void _load_resource_task3(TKEY d);
@@ -64,6 +66,7 @@ void gc_gb_sexp();
 void
 init_load_resource()
 {
+
 	memset(&res_extend,0,sizeof(SYS_QUEUE));
 	res_extend.flags = QF_STACK;
 	res_extend.gc_func = gc_lrw;
@@ -72,6 +75,14 @@ init_load_resource()
 	res_extend.pri = PRI_FETCH;
 	setup_queue(&res_extend);
 
+	memset(&res_q0,0,sizeof(SYS_QUEUE));
+	res_q0.flags = QF_STACK;
+	res_q0.gc_func = gc_lrw;
+	res_q0.gc_get = gc_lrw_get;
+	res_q0.key_func = _load_resource_task0;
+	res_q0.pri = PRI_FETCH;
+	setup_queue(&res_q0);
+
 	memset(&res_q1,0,sizeof(SYS_QUEUE));
 	res_q1.flags = QF_STACK;
 	res_q1.gc_func = gc_lrw;
@@ -350,13 +361,22 @@ void
 _res_end_op_extend(LOAD_RESOURCE_WORK * w,int err,RESOURCE * r)
 {
 LOAD_RESOURCE_WORK * wp;
-
+L_CHAR * f, * g;
 
 	w->err = err;
+	if ( err < 0 ) {
+		f = get_url_str2(&w->url);
+		if ( f ) {
+			g = ll_copy_str(f,22);
+			purge_rcache(g);
+			d_f_ree(g);
+		}
+	}
 	if ( r ) {
 		r->h.flags |= RF_LOAD_EXTEND;
 		r->h.flags &= ~RF_LOAD_EXTEND_PROC;
 		wakeup_task((int)r);
+		rcache_invoke();
 	}
 	wakeup_task((int)w->wup);
 }
@@ -378,12 +398,21 @@ void
 _res_end_op(LOAD_RESOURCE_WORK * w,int err,RESOURCE * r)
 {
 LOAD_RESOURCE_WORK * wp;
-
+L_CHAR * f, * g;
 
 	w->err = err;
+	if ( err < 0 ) {
+		f = get_url_str2(&w->url);
+		if ( f ) {
+			g = ll_copy_str(f,22);
+			purge_rcache(g);
+			d_f_ree(g);
+		}
+	}
 	if ( r ) {
 		r->h.flags |= RF_COMPLETE;
 		wakeup_task((int)r);
+		rcache_invoke();
 	}
 	wakeup_task((int)w->wup);
 	_release_first_list(w);
@@ -402,12 +431,21 @@ void
 _res_end_op_2(LOAD_RESOURCE_WORK * w,int err,RESOURCE * r)
 {
 LOAD_RESOURCE_WORK * wp;
-
+L_CHAR * f,* g;
 
 	w->err = err;
+	if ( err < 0 ) {
+		f = get_url_str2(&w->url);
+		if ( f ) {
+			g = ll_copy_str(f,22);
+			purge_rcache(g);
+			d_f_ree(g);
+		}
+	}
 	if ( r ) {
 		r->h.flags |= RF_COMPLETE;
 		wakeup_task((int)r);
+		rcache_invoke();
 	}
 	wakeup_task((int)w->wup);
 	_release_first_list(w);
@@ -835,7 +873,7 @@ ss_printf("\n");
 					gblisp_top_env0));
 			set_env(env,l_string(std_cm,"__resource"),
 				get_ptr(rr,0));
-			last_tag = wp->data_result;
+			rr->h.data = last_tag = wp->data_result;
 			for ( ; get_type(last_tag) == XLT_PAIR &&
 				get_type(cdr(last_tag)) != XLT_NULL ;
 				last_tag = cdr(last_tag) );
@@ -851,7 +889,9 @@ ss_printf("\n");
 			}
 			ret = eval(env,car(last_tag));
 			if ( get_type(ret) == XLT_ERROR ) {
-
+ss_printf("ERROR BODY 1\n");
+print_sexp(s_stdout,rr->h.data,0);
+ss_printf("\n");
 				set_init_status_error(rr,ret);
 				res_end_op(wp,LRWE_ERROR_BODY_EVAL,rr);
 				goto err_end;
@@ -905,6 +945,15 @@ ss_printf("\n");
 }
 
 
+void
+_resource_fix_wait(RESOURCE * r)
+{
+	for ( ; r->h.flags & RF_FIX ; ) {
+		sleep_task((int)r,res_lock);
+		lock_task(res_lock);
+	}
+}
+
 void
 _lr_extend_insert(int ses,LOAD_RESOURCE_WORK * w,L_CHAR * key)
 {
@@ -928,6 +977,7 @@ RESOURCE_TYPE_LIST * rt;
 				(RF_LOAD_EXTEND|RF_LOAD_EXTEND_PROC) ){
 			goto free_next_in_lock;
 		}
+		_resource_fix_wait(wp1->result);
 		wp1->result->h.flags |= RF_LOAD_EXTEND_PROC;
 		unlock_task(res_lock,"_lr_option_insert");
 
@@ -980,6 +1030,142 @@ int ret;
 	return ret;
 }
 
+void
+rs_cache_setup_c_unit(int ses,URL * u,COORDINATE_UNIT * cu)
+{
+L_CHAR * url;
+int ret;
+RESOURCE * rr;
+RS_BUF rsb;
+	url = ll_copy_str(get_url_str2(u),22);
+	setup_c_unit(cu,-1,url,0,0);
+
+	if ( cu->system )
+		goto end;
+
+	setup_c_unit(cu,ses,url,0,0);
+
+end:
+	d_f_ree(url);
+}
+
+
+void
+_load_resource_task0(TKEY d)
+{
+int ses;
+LOAD_RESOURCE_WORK * wp;
+XL_INTERPRETER * xli;
+L_CHAR * key;
+SYS_QUEUE * que;
+int i;
+XLISP_ENV * env;
+RS_BUF rsb;
+XL_SEXP * ret, * last_tag;
+
+	que = (SYS_QUEUE *)GET_TKEY(d);
+	key = touch_qkey(que);
+	if ( key == 0 )
+		return;
+
+	xli = new_xl_interpreter();
+	xli->a_type = XLA_SELF;
+	setup_i(xli);
+
+
+	ses = open_session(SEST_DIRECT);
+	for ( ; ; ) {
+		gc_push(0,0,"_load_resource_task1");
+
+		lock_task(res_lock);
+		unlock_task(res_lock,"_load_resource_task1");
+
+		wp = delete_queue(&res_q0,0,0,0);
+		if ( wp == 0 ) {
+			gc_pop(0,0);
+			break;
+		}
+		if ( wp->que_next )
+			er_panic("resource");
+		switch ( wp->option ) {
+		case LRO_FULL:
+			if ( get_rs_resource_cache(&rsb,
+					get_url_str2(&wp->url)) < 0 ) {
+/*
+ss_printf("GET EMPTY %ls\n",get_url_str2(&wp->url));
+*/
+				insert_queue(&res_q1,wp,1);
+				break;;
+			}
+/*
+ss_printf("GET SUCCESS %ls\n",get_url_str2(&wp->url));
+*/
+			if ( rsb.flags & RSBF_MAP ) {
+ss_printf("MAP--------\n");
+				rs_cache_setup_c_unit(
+					ses,
+					&rsb.r->map.src,
+					&rsb.r->map.cu_src);
+				rs_cache_setup_c_unit(
+					ses,
+					&rsb.r->map.dest,
+					&rsb.r->map.cu_dest);
+			}
+			wp->result = rsb.r;
+			wp->data_result = rsb.sexp;
+			if ( gv_resource_env[rsb.r->h.type] ) {
+				env = new_env(
+					new_env_pair(gv_resource_env[rsb.r->h.type],
+						gblisp_top_env0));
+				set_env(env,l_string(std_cm,"__resource"),
+					get_ptr(rsb.r,0));
+				last_tag = wp->data_result;
+				for ( ; get_type(last_tag) == XLT_PAIR &&
+					get_type(cdr(last_tag)) != XLT_NULL ;
+					last_tag = cdr(last_tag) );
+				if ( get_type(last_tag) != XLT_PAIR ) {
+					set_init_status_error(rsb.r,0);
+					res_end_op(wp,LRWE_ERROR_NO_ROOT_XML,rsb.r);
+ss_printf("--------- CACHE ERROR %i\n",rsb.r->h.type);
+print_sexp(s_stdout,wp->data_result,0);
+ss_printf("\n");
+print_sexp(s_stdout,ret,0);
+ss_printf("\n");
+					break;
+				}
+				ret = eval(env,car(last_tag));
+				if ( get_type(ret) == XLT_ERROR ) {
+ss_printf("ERROR BODY 2\n");
+print_sexp(s_stdout,rsb.sexp,0);
+ss_printf("\n\n");
+print_sexp(s_stdout,ret,0);
+ss_printf("\n");
+
+					set_init_status_error(rsb.r,ret);
+					res_end_op(wp,LRWE_ERROR_BODY_EVAL,rsb.r);
+					break;
+				}
+			}
+			res_end_op(wp,LRWE_OK,wp->result);
+			break;
+		case LRO_UNIT:
+			insert_queue(&res_q1,wp,1);
+			break;
+		default:
+			er_panic("task0");
+		}
+	next:
+		gc_pop(0,0);
+	}
+
+	release_qkey(que,key);
+	d_f_ree(key);
+
+	close_session(ses);
+	close_self_interpreter();
+
+}
+
 
 void
 _load_resource_task1(TKEY d)
@@ -1329,7 +1515,7 @@ ss_printf("IN (%i) %ls\n",get_tid(),entry);
 	wp = w;
 	for ( ; wp ; wp = wp->req_next ) {
 		if ( wp->err == LRWE_HANDLING )
-			insert_queue(&res_q1,wp,1);
+			insert_queue(&res_q0,wp,1);
 		else if ( wp->err == LRWE_DUP )
 			wp->err = LRWE_HANDLING;
 	}
@@ -1375,8 +1561,8 @@ retry:
 	lock_task(res_lock);
 	wp = w;
 	for ( ; wp ; wp = wp->req_next )
-		if ( wp->err == LRWE_HANDLING ) {
-			sleep_task((int)w,res_lock);
+		if ( wp->err == LRWE_HANDLING || wp->err == LRWE_DUP ) {
+			sleep_task((int)w->wup,res_lock);
 			goto retry;
 		}
 slp2:
@@ -1524,3 +1710,57 @@ LOAD_RESOURCE_WORK * wp;
 	return 0;
 }
 
+void
+free_resource_entry(RESOURCE * r)
+{
+	printf("free_Resource_entry\n");
+	return;
+}
+
+void
+_resource_cache_invoke(RESOURCE * r)
+{
+RS_BUF b;
+	b.r = r;
+	b.sexp = r->h.data;
+
+	set_rs_resource_cache(&b);
+
+	r->h.data = 0;
+}
+
+
+void
+resource_cache_invoke()
+{
+int i;
+RESOURCE * r;
+extern RESOURCE ** res_no_hash_table;
+
+printf("RESORCE CACHE INVOKE\n");
+	for ( i = 0 ; i < RES_HASH_SIZE ; ) {
+		lock_task(res_lock);
+		for ( r = res_no_hash_table[i] ; r ; r = r->h.no_next )
+			if ( (r->h.flags & (RF_CACHE|RF_COMPLETE|RF_LOAD_EXTEND_PROC))
+					== RF_COMPLETE )
+				break;
+		if ( r == 0 ) {
+			unlock_task(res_lock,"resource_cache_invoke");
+			i ++;
+			continue;
+		}
+		r->h.flags |= RF_FIX;
+		unlock_task(res_lock,"reource_cache_invoke");
+
+		_resource_cache_invoke(r);
+
+		lock_task(res_lock);
+		r->h.flags &= ~RF_FIX;
+		r->h.flags |= RF_CACHE;
+		wakeup_task((int)r);
+		unlock_task(res_lock,"resource_cache_invoke");
+	}
+	flush_rs_cache();
+}
+
+
