@@ -29,6 +29,7 @@
 #include	"machine/param_gbview.h"
 #include	"lock_level.h"
 #include	"proxy.h"
+#include	"rcache.h"
 
 extern GBVIEW_ENV env;
 extern char* http_client;
@@ -420,6 +421,92 @@ typemissmatch:
 			0));
 }
 
+XL_SEXP *
+xl_set_cache(XLISP_ENV * env,XL_SEXP * s,XLISP_ENV * a,XL_SYM_FIELD * sf)
+{
+L_CHAR * _resource,* _routing;
+int flags;
+	_resource = get_sf_attribute(
+				sf,
+				l_string(std_cm,"resource"));
+	_routing = get_sf_attribute(
+				sf,
+				l_string(std_cm,"routing"));
+	flags = get_rcache_flags();
+	if ( _resource ) {
+		flags &= ~RCF_RESOURCE_MASK;
+		if ( l_strcmp(_resource,l_string(std_cm,"off")) == 0 ) {
+		}
+		else if ( l_strcmp(_resource,l_string(std_cm,"w")) == 0 ) {
+			flags |= RCF_RESOURCE;
+		}
+		else if ( l_strcmp(_resource,l_string(std_cm,"r/w")) == 0 ) {
+			flags |= RCF_RESOURCE|RCF_RESOURCE_READ;
+		}
+		else	goto err;
+	}
+	if ( _routing ) {
+		flags &= ~RCF_ROUTING_MASK;
+		if ( l_strcmp(_routing,l_string(std_cm,"off")) == 0 ) {
+		}
+		else if ( l_strcmp(_routing,l_string(std_cm,"w")) == 0 ) {
+			flags |= RCF_ROUTING;
+		}
+		else if ( l_strcmp(_routing,l_string(std_cm,"r/w")) == 0 ) {
+			flags |= RCF_ROUTING|RCF_ROUTING_READ;
+		}
+		else 	goto err;
+	}
+	enable_rcache(flags);
+	return 0;
+err:
+	return get_error(
+		s->h.file,
+		s->h.line,
+		XLE_PROTO_INV_PARAM,
+		l_string(std_cm,"gv-set-cache"),
+		list(	n_get_string(
+		"invalid attribute"),
+			0));
+
+}
+
+
+XL_SEXP *
+xl_get_cache(XLISP_ENV * env,XL_SEXP * s)
+{
+int flags;
+XL_SEXP * _routing, * _resource;
+	flags = get_rcache_flags();
+	if ( flags & RCF_ROUTING_READ ) {
+		_routing = n_get_string("r/w");
+	}
+	else if ( flags & RCF_ROUTING ) {
+		_routing = n_get_string("w");
+	}
+	else {
+		_routing = n_get_string("off");
+	}
+	if ( flags & RCF_RESOURCE_READ ) {
+		_resource = n_get_string("r/w");
+	}
+	else if ( flags & RCF_RESOURCE ) {
+		_resource = n_get_string("w");
+	}
+	else {
+		_resource = n_get_string("off");
+	}
+	return List(
+		n_get_symbol("cache"),
+		List(n_get_symbol("routing"),
+			_routing,
+			-1),
+		List(n_get_symbol("resource"),
+			_resource,
+			-1),
+		-1);
+}
+
 
 XL_SEXP *
 xl_status_param(XLISP_ENV * env,XL_SEXP * s)
@@ -434,6 +521,8 @@ STREAM * st;
 		init_parse(st,l_string(std_cm,"gbview parameter file"),0));
 }
 
+
+
 void
 write_proxy_server_list(STREAM * st,XL_SEXP * sl)
 {
@@ -475,6 +564,35 @@ XL_SEXP * sl;
 	gc_pop(0,0);
 }
 
+void
+write_cache_info(STREAM * st)
+{
+int flags;
+char * _resource, * _routing;
+	flags = get_rcache_flags();
+	if ( flags & RCF_RESOURCE_READ ) {
+		_resource = "r/w";
+	}
+	else if ( flags & RCF_RESOURCE ) {
+		_resource = "w";
+	}
+	else {
+		_resource = "off";
+	}
+	if ( flags & RCF_ROUTING_READ ) {
+		_routing = "r/w";
+	}
+	else if ( flags & RCF_ROUTING ) {
+		_routing = "w";
+	}
+	else {
+		_routing = "off";
+	}
+	s_printf(st,"<gv-set-cache routing=\"%s\" resource = \"%s\"/>\n",
+		_routing,
+		_resource);
+}
+
 XL_SEXP *
 xl_write_status(XLISP_ENV * e,XL_SEXP * s)
 {
@@ -494,6 +612,7 @@ STREAM * st;
 	s_printf(st,"<gv-resource rid=\"4\">\n");
 
 	write_proxy_info(st);
+	write_cache_info(st);
 
 	s_printf(st,"<initial-query type=\"direct\">\n");
 	print_sexp(st,setup_initial_query,PF_XML|PF_INDENT);
@@ -504,6 +623,7 @@ STREAM * st;
 /*
 	s_printf(st,"<http-client>%s </http-client>\n",http_client);
 */
+
 	s_printf(st,"</gv-resource>\n\n");
 
 	s_close(st);
@@ -564,6 +684,12 @@ int i;
 	set_env(e,l_string(std_cm,"gv-reset-status"),
 		get_func_prim(xl_reset_status,
 			FO_APPLICATIVE,0,1,1));
+	set_env(e,l_string(std_cm,"gv-set-cache"),
+		get_func_prim(xl_set_cache,
+			FO_APPLICATIVE,0,1,1));
+	set_env(e,l_string(std_cm,"gv-get-cache"),
+		get_func_prim(xl_get_cache,
+			FO_APPLICATIVE,0,1,1));
 
 	set_gv_sp_resource(RID_INIT,e);
 
@@ -802,10 +928,7 @@ struct dirent * d;
 			exit(1);
 		}
 	} else {
-		char* _dir = change_delim_str(get_peripheral_path());
-		dd = opendir(_dir);
-		if ( _dir != get_peripheral_path() )
-			d_f_ree(_dir);
+		dd = u_opendir(get_peripheral_path());
 		if ( dd == 0 ) {
 			fprintf(stderr,"cannot open the path %s\n",
 				get_peripheral_path());
@@ -824,14 +947,8 @@ struct dirent * d;
 				continue;
 
 			sprintf(file,"%s%s",get_peripheral_path(),d->d_name);
-			st = s_open_file(file,O_RDONLY);
-			if ( st == 0 )
-				continue;
-			lst = init_parse(st,
-					l_string(std_cm,"preference.xl"),
-					l_string(std_cm,"preference.xl"));
-
 			env = new_env(gblisp_top_env1);
+			lst = load_file(env,0,2,l_string(std_cm,file),0,0,0,0,0);
 			for ( ; get_type(lst) == XLT_PAIR ; lst = cdr(lst) ) {
 				gc_push(0,0,"loop2");
 				ret = eval(env,car(lst));
