@@ -897,27 +897,22 @@ int ret_f;
 		lr = get_lr(0,lmp,0);
 
 		for ( lr1 = lr ; lr1 ; lr1 = lr1->next ) {
-
+/*
 ss_printf("retrieve loop %ls\n",lr1->lump_crd);
-
+*/
 			ret_f = 0;
 			lmp = load_meta(ses,"lump",lr1->lump_crd);
-set_cpu_msg(101);
 			if ( get_type(lmp) == XLT_ERROR )
 				continue;
 			init_lump_opt(&opt);
 			lr2 = get_lr(&opt,lmp,0);
 			free_lr(lr2);
 			if ( opt.lump == 0 )
-{ss_printf("E1\n");
 				goto next2;
-}
 			lock_task(radar_lock);
 			l = _search_radar_lump(lr1->lump_crd);
 			if ( l )
-{ss_printf("E2\n");
 				goto next1;
-}
 			unlock_task(radar_lock,"retrieve_lump");
 			l = setup_radar_lump(ses,lr1->lump_crd,opt.lump);
 			if ( l == 0 )
@@ -930,7 +925,6 @@ set_cpu_msg(101);
 				unlock_task(radar_lock,"retrieve_lump");
 				call_gv_event(RID_RADAR,
 					l_string(std_cm,"delete"));
-ss_printf("E3\n");
 				goto end1;
 			}
 			if ( _insert_radar_lump_list(l) < 0 ) {
@@ -939,12 +933,10 @@ ss_printf("E3\n");
 				unlock_task(radar_lock,"retrieve_lump");
 				call_gv_event(RID_RADAR,
 					l_string(std_cm,"delete"));
-ss_printf("E4\n");
 				goto end1;
 			}
 			ret = 1;
 			ret_f = 1;
-ss_printf("OK\n");
 		next1:
 			_insert_radar_lump(c,l);
 			unlock_task(radar_lock,"retrieve_lump");
@@ -966,7 +958,6 @@ ss_printf("OK\n");
 	end0:
 		d_f_ree(obj);
 		d_f_ree(crd);
-ss_printf("EEE\n");
 	}
 	return ret;
 }
@@ -1612,7 +1603,6 @@ free_radar_matrix(void)
 {
 	lock_task(radar_lock);
 	_free_radar_matrix(rc.matrix);
-ss_printf("FREE RM\n");
 	rc.matrix = 0;
 	unlock_task(radar_lock,"free_radar_matrix");
 }
@@ -1674,10 +1664,7 @@ void
 _clean_radar_matrix()
 {
 	if ( __clean_radar_matrix(rc.matrix) == 0 )
-{
-ss_printf("CREAN\n");
 		rc.matrix = 0;
-}
 	_clean_top_of_matrix();
 }
 
@@ -1928,8 +1915,9 @@ URL u;
 LOAD_RESOURCE_WORK * w;
 
 	w = 0;
-if ( l_strcmp(crd,obj) == 0 )
+/*if ( l_strcmp(crd,obj) == 0 )
 printf("ins_url %s\n",n_string(std_cm,crd));
+*/
 	rlock_base_url("_insert_radar_url");
 /*
 printf("R1\n");
@@ -1979,9 +1967,8 @@ c->minrect.br.y);
 	free_url(&u);
 	if ( r2 == 0 )
 		goto flush_end;
-/*
-printf("AA\n");
-*/
+
+
 	c = new_radar_crd();
 	c->crd_path = ll_copy_str(crd,1518);
 	c->obj_path = ll_copy_str(obj,1519);
@@ -2029,7 +2016,7 @@ end:
 
 
 int
-remake_cache(int ses,int type,WF_ID wfid,WF_ID next)
+remake_cache(int ses,int type,WF_ID current,WF_ID next)
 #define RCT_CHANGE	1
 #define RCT_DELETE	2
 {
@@ -2040,18 +2027,17 @@ URL u;
 L_CHAR * new;
 RESOURCE *draw;
 int i,j;
-
+WF_ID wfid;
 
 set_cpu_msg(2501);
-
+/*
 ss_printf("NEXT %i\n",next);
-
-
+*/
+	if ( current )
+		wf_set_current_now(current);
 	wfid = wf_get_current();
 	if ( next )
 		wf_set_current_target(next);
-if ( env.flame == 0 )
-er_panic("???");
 
 	switch ( type ) {
 	case RCT_CHANGE:
@@ -2061,9 +2047,7 @@ er_panic("???");
 	case RCT_DELETE:
 		if ( rc.win_delete_lock == 0 &&
 				wf_check_last(wfid) == 0 )
-{ss_printf("REMAKE FREE\n");
 			free_win_flame(wfid);
-}
 		else return 0;
 		wf_wait_stable();
 		break;
@@ -2078,10 +2062,6 @@ er_panic("???");
 
 	if ( draw == 0 ) {
 
-
-ss_printf("DRAW ZERO *************************************** DRAW ZERO\n");
-if ( env.flame == 0 )
-er_panic("stop");
 		lock_task(radar_lock);
 		if ( rc.base_url )
 			d_f_ree(rc.base_url);
@@ -2160,19 +2140,21 @@ er_panic("stop");
 		if ( c->base_url )
 			d_f_ree(c->base_url);
 		c->base_url = 0;
+/*
 ss_printf("ins %s\n",n_string(std_cm,c->obj_path));
+*/
 		if ( c->flags & RCF_INMATRIX ) 
 			insert_radar_matrix(c);
 	}
 
 
-
+/*
 ss_printf("REMAKE %ls --> ",rc.base_url);
-
+*/
 	d_f_ree(rc.base_url);
-
+/*
 ss_printf("%ls\n",new);
-
+*/
 	rc.base_url = ll_copy_str(new,1523);
 
 end_flush:
@@ -2259,7 +2241,6 @@ RESOURCE * res;
 		switch ( cl->sts ) {
 		case CLS_KEEP:
 			cl->sts = CLS_REMOVE;
-ss_printf("REMOVE 6 ======================\n");
 			break;
 		case CLS_REMOVE:
 			break;
@@ -2333,9 +2314,9 @@ AVT_NODE * reso_tree;
 		for ( n = lst->mx->list ; n ; n = n->matrix_next ) {
 			c = n->crd;
 			c->lock ++;
-
+/*
 ss_printf("CMP BEFOR %ls %lf\n",c->crd_path,(double)c->resolution);
-
+*/
 			if ( cross_rect_rect(&c->minrect,r) == 0 )
 				continue;
 
@@ -2343,9 +2324,9 @@ ss_printf("CMP BEFOR %ls %lf\n",c->crd_path,(double)c->resolution);
 				continue;
 			if ( c->crd_r && (c->crd_r->h.flags & RF_INDICATE) == RF_I_OFF )
 				continue;
-
-ss_printf("CMP CACHE %ls %lf\n",c->crd_path,(double)c->resolution);
 /*
+ss_printf("CMP CACHE %ls %lf\n",c->crd_path,(double)c->resolution);
+
 ss_printf("MINRECT (%f %f) = (%f %f)\n",
 c->minrect.tl.x,
 c->minrect.tl.y,
@@ -2377,7 +2358,6 @@ r->br.y);
 				clp->sts = CLS_INSERT;
 				a = d_alloc(sizeof(*a),1252);
 				a->data = clp;
-ss_printf("> %x\n",clp);
 				avt_insert(tree,a,cmp_wf_crd);
 			}
 		}
@@ -2534,9 +2514,7 @@ CRD_LIST * cl;
 	if ( rc.win_delete_lock )
 		return 0;
 	if ( wf_check_last(cl->wfid) == 0 )
-{ss_printf("DELETE FUNC FREE\n");
 		free_win_flame(cl->wfid);
-}
 	return 0;
 }
 
@@ -2579,7 +2557,6 @@ CRD_LIST * cl;
 
 	if ( cl->wfid && w->target->wfid ) {
 		cl->sts = CLS_REMOVE;
-ss_printf("REMOVE 1 ======================\n");
 		cl->crd = 0;
 	}
 	else if ( cl->wfid ) {
@@ -2647,8 +2624,9 @@ h_relation_master(AVT_NODE * a,H_RELATION_WORK * w)
 {
 CRD_LIST * cl;
 	cl = a->data;
-
+/*
 ss_printf("REL %x %x\n",cl,cl->crd);
+*/
 	if ( cl == w->target )
 		return 0;
 	if ( cl->sts == CLS_NONE ||
@@ -2662,7 +2640,6 @@ ss_printf("REL %x %x\n",cl,cl->crd);
 
 	if ( w->target->wfid ) {
 		w->target->sts = CLS_REMOVE;
-ss_printf("REMOVE 2 ====================== %ls %ls\n",cl->crd->crd_path,w->target->crd->crd_path);
 		return 1;
 	}
 	w->target->sts = CLS_NONE;
@@ -2688,7 +2665,6 @@ CRD_LIST * cl;
 
 	if ( cl->wfid ) {
 		cl->sts = CLS_REMOVE;
-ss_printf("REMOVE 3 ======================\n");
 		return 1;
 	}
 	cl->sts = CLS_NONE;
@@ -2951,6 +2927,7 @@ HEURISTICS_PARAM hw;
 BASE_PRI_WORK bw;
 
 	lock_task(radar_lock);
+/*
 ss_printf("WIN_MANAGE (%f %f)(%f %f) %f - %ls\n",
 r->tl.x,
 r->tl.y,
@@ -2960,7 +2937,7 @@ reso,
 rc.base_url
 );
 ss_printf(">>> %i\n",check_matrix_size());
-
+*/
 	_rlock_base_url("win_manage");
 
 retry:
@@ -3050,9 +3027,9 @@ ss_printf("NEXT %ls\n",bw.cl->crd->crd_path);
 	lock_task(radar_lock);
 
 	_unlock_base_url();
-
+/*
 ss_printf("WIN_MANAGE END\n");
-
+*/
 	rc.win_reso_max = reso;
 	rc.win_rect = *r;
 	unlock_task(radar_lock,"win_manage");
@@ -3157,7 +3134,6 @@ query_trigger(int fn_flag)
 	rc_task_lock(TM_TRIG);
 	_query_trigger(fn_flag);
 	rc_task_unlock(TM_TRIG);
-printf("***** query_trigger end\n");
 }
 
 void
@@ -3177,7 +3153,6 @@ void
 query_trigger2()
 {
 
-printf("***** query_trigger2\n");
 	rc_task_lock(TM_TRIG);
 	_query_trigger2();
 	rc_task_unlock(TM_TRIG);
@@ -3916,7 +3891,6 @@ GB_POINT p;
 	get_url2(&u,target,1565);
 
 	c = load_resource(ses,&u,Q_PRI_RADAR);
-set_cpu_msg(107);
 
 
 	free_url(&u);
@@ -4097,7 +4071,6 @@ individual:
 	if ( rc.base_url )
 		d_f_ree(rc.base_url);
 	_free_radar_matrix(rc.matrix);
-ss_printf("INDIVI\n");
 	rc.matrix = 0;
 	rc.base_url = ll_copy_str(_crd,1529);
 	get_center_reso(&p,&reso,&_r);
@@ -4353,6 +4326,8 @@ printf("insert_task del %i\n",get_tid());
 				unlock_task(radar_lock,
 					"beam_insert_task");
 			}
+if ( rc.base_url == 0 )
+er_panic("b");
 		}
 		unlock_base_url();
 
@@ -4927,9 +4902,9 @@ printf("min_route task\n");
 */
 			unlock_task(radar_lock,"min_route_task");
 			min_ret = min_route_wf(ses);
-
+/*
 printf("min_ret = %i\n",min_ret);
-
+*/
 			lock_task(radar_lock);
 /*
 			_rc_task_unlock(TM_MIN);
@@ -5091,7 +5066,7 @@ set_cpu_msg(23);
 
 		rc_task_unlock(TM_WMAN);
 
-		if ( ret <= 0 ) {
+		if ( ret >= 0 ) {
 			if ( rc_task_lock_check(TM_REMK) == 0 ) {
 				rc_task_lock(TM_REMK);
 				rr = remake_cache(ses,RCT_CHANGE,ret,next);
