@@ -317,6 +317,10 @@ typedef struct res_header {
 #define RF_DELETE		0x00000001
 #define RF_DELETE_ACCEPT	0x00000002
 
+#define RF_FIX			0x00000004
+
+#define RF_CACHE		0x00000010
+
 #define RF_COMPLETE		0x00000100
 #define RF_LOAD_EXTEND		0x00000200
 #define RF_LOAD_EXTEND_PROC	0x00000400
@@ -325,6 +329,7 @@ typedef struct res_header {
 #define RF_I_OFF		0x00000000
 #define RF_I_ON			0x01000000
 #define RF_I_AUTO		0x02000000
+	unsigned int	modify;
 	GB_RECT		minrect;
 
 	XL_SEXP *	meta;
@@ -341,6 +346,8 @@ typedef struct res_header {
 	NOTE *		notes;
 	REAL1		visible_resolution;
 	COORDINATE_UNIT cu;
+
+	XL_SEXP *	data;
 } RES_HEADER;
 
 
