@@ -928,7 +928,8 @@ printf("ERR (%i) 4\n",get_tid());
 		case -1:
 		case 0:
 			s_info->flags |= XoH_F_ERR|XoH_F_RECV_LAST;
-printf("ERR (%i) 5\n",get_tid());
+printf("ERR (%i) 5 %i %i\n",get_tid(),ret,errno);
+perror("EEE");
 			break;
 		case 1:
 			break;
