@@ -41,6 +41,11 @@ typedef struct con_prefix {
 	L_CHAR *		cmd;
 } CON_PREFIX;
 
+typedef struct con_agent_initial {
+	struct con_agent_initial *	next;
+	int				ses;
+} CON_AGENT_INITIAL;
+
 typedef struct con_agent {
 	struct con_agent *	next;
 	L_CHAR *		name;
@@ -56,6 +61,7 @@ typedef struct con_agent {
 	CON_PREFIX *		prefix;
 	L_CHAR *		login_mode;
 	int			lock;
+	CON_AGENT_INITIAL *	initial_list;
 } CON_AGENT;
 
 typedef struct con_server {
@@ -80,9 +86,24 @@ typedef struct con_session {
 
 	int			flags;
 #define CSF_CLOSE		0x00000001
+#define CSF_PROXY_INIT		0x00000002
 	int			access_cnt;
+
+	int			(*pre_func)();
+	XL_SEXP *		(*pre_proxy)();
+	void			(*pre_gc)();
+	void			(*pre_free)();
+	void *			pre_work;
 } CON_SESSION;
 
+typedef struct nproxy_direct_server {
+	struct nproxy_direct_server *	next;
+	L_CHAR *			server;
+} NPROXY_DIRECT_SERVER;
+
+void gc_gb_sexp();
+
+
 int session_id;
 
 SEM session_lock;
@@ -91,6 +112,9 @@ CON_SESSION *session_list;
 CON_IND_AGENT *	ind_agent_list;
 
 char * setagent_msg;
+int native_proxy_session;
+URL native_proxy_url;
+NPROXY_DIRECT_SERVER *	np_server_list;
 
 
 void agent_timeout_task();
@@ -232,6 +256,54 @@ CON_PREFIX * pf1;
 	}
 }
 
+
+CON_AGENT_INITIAL *
+_search_con_agent_initial(CON_AGENT * a,int ses)
+{
+CON_AGENT_INITIAL * ai;
+	for ( ai = a->initial_list ; ai ; ai = ai->next )
+		if ( ai->ses == ses )
+			return ai;
+	return 0; 
+}
+
+void
+_insert_con_agent_initial(CON_AGENT * a,int ses)
+{
+CON_AGENT_INITIAL * ai;
+	ai = d_alloc(sizeof(*ai),33);
+	ai->ses = ses;
+	ai->next = a->initial_list;
+	a->initial_list = ai;
+}
+
+void
+_free_con_agent_initial(CON_AGENT_INITIAL * ai)
+{
+CON_AGENT_INITIAL * ai2;
+	for ( ; ai ; ) {
+		ai2 = ai;
+		ai = ai->next;
+		d_f_ree(ai2);
+	}
+}
+
+
+void
+_free_con_agent_initial_one(CON_AGENT * a,int ses)
+{
+CON_AGENT_INITIAL * ai, ** aip;
+	for ( aip = &a->initial_list ; *aip ; aip = &(*aip)->next ) {
+		ai = *aip;
+		if ( ai->ses != ses )
+			continue;
+		*aip = ai->next;
+		d_f_ree(ai);
+		return;
+	}
+}
+
+
 CON_AGENT *
 _search_con_agent(CON_SERVER * server,L_CHAR * name)
 {
@@ -360,6 +432,7 @@ CON_IND_AGENT * ia;
 	a->server = server;
 	a->prefix = 0;
 	a->login_mode = 0;
+	a->initial_list = 0;
 	a->next = server->agent;
 	a->timeout = ATT_IDLE;
 	server->agent = a;
@@ -372,6 +445,7 @@ _free_con_agent(CON_AGENT * a)
 	_free_con_prefix(a->prefix);
 	d_f_ree(a->name);
 	d_f_ree(a);
+	_free_con_agent_initial(a->initial_list);
 }
 
 CON_SERVER *
@@ -469,6 +543,11 @@ CON_SESSION * s;
 	s->lock = 0;
 	s->sl_next = 0;
 	s->type = type;
+	s->pre_func = 0;
+	s->pre_proxy = 0;
+	s->pre_gc = 0;
+	s->pre_free = 0;
+	s->pre_work = 0;
 	session_id ++;
 	if ( session_id <= 0 )
 		session_id = 1;
@@ -559,6 +638,31 @@ CON_AGENT * a;
 }
 
 
+
+int
+_set_pre_func(
+	CON_SESSION * ses,
+	int (*pre_func)(),
+	XL_SEXP * (*pre_proxy)(),
+	void (*pre_gc)(),
+	void (*pre_free)(),
+	void * pre_work)
+{
+CON_AGENT * a;
+CON_SERVER * s;
+	if ( ses->pre_free )
+		(*ses->pre_free)(ses->pre_work);
+	ses->pre_func = pre_func;
+	ses->pre_proxy = pre_proxy;
+	ses->pre_gc = pre_gc;
+	ses->pre_work = pre_work;
+	for ( s = server_list ; s ; s = s->next )
+		for ( a = s->agent ; a ; a = a->next )
+			_free_con_agent_initial_one(a,ses->id);
+	ses->flags &= ~CSF_PROXY_INIT;
+	return 0;
+}
+
 int
 open_session(int type)
 {
@@ -588,6 +692,10 @@ CON_SESSION ** sp, * s;
 			lock_task(session_lock);
 		}
 		s->lock = 1;
+
+		if ( s->pre_free )
+			(*s->pre_free)(s->pre_work);
+
 		if ( s->server )
 			_detouch_server(s);
 		*sp = s->next;
@@ -605,6 +713,98 @@ close_session(int id)
 }
 
 
+typedef struct init_sexp_work {
+	XL_SEXP *		query;
+	XLISP_ENV *		env;
+} INIT_SEXP_WORK;
+
+int
+init_sexp_func(
+	CON_SESSION * ses,
+	CON_AGENT * a,
+	int proxy_ses)
+{
+XL_SEXP * q;
+INIT_SEXP_WORK * w;
+	w = (INIT_SEXP_WORK*)ses->pre_work;
+	for (q = w->query  ;
+			get_type(q) != XLT_PAIR ;
+			q = cdr(q) ) {
+		remote_query(a->iid,w->env,0,car(q));
+	}
+	return 0;
+}
+
+XL_SEXP *
+init_sexp_proxy(CON_SESSION * ses)
+{
+INIT_SEXP_WORK * w;
+	w = (INIT_SEXP_WORK*)ses->pre_work;
+	if ( w == 0 )
+		return 0;
+	return List(
+		n_get_symbol("SetSessionEnv"),
+		List(n_get_symbol("quote"),
+			w->query,
+			-1),
+		-1);
+}
+
+
+void
+init_sexp_free(
+	INIT_SEXP_WORK * w)
+{
+	d_f_ree(w);
+}
+
+void
+init_sexp_gc(
+	INIT_SEXP_WORK * w)
+{
+	gc_gb_sexp(w->query);
+	gc_gblisp_env(w->env);
+}
+
+int
+_set_initial_sexp(int id,XLISP_ENV * env,XL_SEXP * s)
+{
+CON_SESSION * ses;
+INIT_SEXP_WORK * w;
+	ses = _search_con_session(id);
+	if ( ses == 0 )
+		return -1;
+	if ( s ) {
+		w = d_alloc(sizeof(*w),22);
+		w->env = env;
+		w->query = s;
+		_set_pre_func(
+			ses,
+			init_sexp_func,
+			init_sexp_proxy,
+			init_sexp_gc,
+			init_sexp_free,
+			w);
+	}
+	else {
+		_set_pre_func(ses,0,0,0,0,0);
+	}
+	return 0;
+}
+
+int
+set_initial_sexp(int id,XLISP_ENV * env,XL_SEXP * s)
+{
+int ret;
+	lock_task(session_lock);
+	ret = _set_initial_sexp(id,env,s);
+	unlock_task(session_lock,"set_initial_sexp");
+	return ret;
+}
+
+
+
+
 int
 connect_server(XLISP_ENV * env,URL * u)
 {
@@ -653,7 +853,8 @@ set_cpu_msg(973);
 
 
 XL_SEXP *
-connect_agent(XLISP_ENV * env,CON_AGENT * a,L_CHAR * login_mode,
+connect_agent(
+	XLISP_ENV * env,CON_AGENT * a,L_CHAR * login_mode,
 		XL_FILE * f,int ln)
 {
 XL_SEXP * ret;
@@ -763,7 +964,13 @@ printf("ret C %i\n",get_tid());
 	unlock_xli_out_iid(a->iid);
 	set_c_timeout(a->iid,ATT_TIMEOUT);
 
+	lock_task(session_lock);
+	_free_con_agent_initial(a->initial_list);
+	a->initial_list = 0;
+	unlock_task(session_lock,"connect_agent");
+
 	send_code_sync(a->iid);
+
 	ret = 0;
 	iid = 0;
 
@@ -954,7 +1161,8 @@ _remote_session(
 	XL_FILE * f,
 	int ln,
 	int session_lock_enable,
-	int ignore_close_flag)
+	int ignore_close_flag,
+	int proxy_ses)
 {
 CON_SESSION * ses;
 CON_SERVER * sv;
@@ -964,6 +1172,7 @@ int retry_cnt;
 L_CHAR * path;
 XL_SEXP * _cmd;
 L_CHAR * agent,* login_mode, * center_cmd;
+
 	if ( a_agent )
 		agent = ll_copy_str(a_agent,123);
 	else	agent = 0;
@@ -974,9 +1183,7 @@ L_CHAR * agent,* login_mode, * center_cmd;
 		center_cmd = ll_copy_str(a_center_cmd,32);
 	else	center_cmd = 0;
 
-set_cpu_msg(90);
 	lock_task(session_lock);
-set_cpu_msg(91);
 
 	ses = _search_con_session(id);
 	if ( ses == 0 ) {
@@ -991,7 +1198,6 @@ set_cpu_msg(91);
 				-1));
 		goto err;
 	}
-set_cpu_msg(92);
 	if ( ignore_close_flag == 0 && ses->flags & CSF_CLOSE ) {
 		unlock_task(session_lock,"remote_session");
 		ret = get_error(
@@ -1007,7 +1213,6 @@ set_cpu_msg(92);
 		sleep_task((int)ses,session_lock);
 		lock_task(session_lock);
 	}
-set_cpu_msg(93);
 	ses->lock = 1;
 	if ( login_mode ) {
 		if ( ses->login_mode )
@@ -1037,8 +1242,6 @@ set_cpu_msg(93);
 		}
 	}
 
-set_cpu_msg(94);
-
 
 	sv = _new_con_server(u,ses);
 
@@ -1073,7 +1276,6 @@ set_cpu_msg(94);
 		ses->working_path = path;
 	}
 
-set_cpu_msg(95);
 
 	_atouch_server(ses,sv);
 
@@ -1109,8 +1311,6 @@ set_cpu_msg(95);
 	}
 
 
-set_cpu_msg(96);
-
 	set_env(env,l_string(std_cm,"SessionPath"),
 		get_string(ses->working_path));
 	if ( u->resource )
@@ -1126,10 +1326,25 @@ retry:
 	ret = connect_agent(env,a,ses->login_mode,f,ln);
 
 
-
-set_cpu_msg(98);
 	if ( get_type(ret) == XLT_ERROR )
 		goto err1;
+
+	read_lock_agent(a);
+
+	lock_task(session_lock);
+	if ( _search_con_agent_initial(a,ses->id) == 0 ) {
+		unlock_task(session_lock,"_remote_session");
+
+		if ( ses->pre_func )
+			(*ses->pre_func)(ses,a,proxy_ses);
+
+		lock_task(session_lock);
+		_insert_con_agent_initial(a,ses->id);
+	}
+	unlock_task(session_lock,"_remote_session");
+
+	read_unlock_agent(a);
+
 	_cmd = cmd;
 
 	for ( ; get_type(_cmd) == XLT_PAIR ; _cmd = cdr(_cmd) ) {
@@ -1140,9 +1355,7 @@ set_cpu_msg(98);
 			goto err1;
 		}
 
-set_cpu_msg(981);
 		read_lock_agent(a);
-set_cpu_msg(982);
 
 
 		ret = remote_query(a->iid,env,0,r);
@@ -1171,6 +1384,7 @@ set_cpu_msg(982);
 			resp->s_file = f;
 			resp->s_line = ln;
 			resp->s_session = ses;
+			resp->s_proxy_ses = proxy_ses;
 			resp->h.func = remote_session_delay;
 			resp->h.check_func = check_remote_session_delay;
 			unlock_mem();
@@ -1180,7 +1394,6 @@ set_cpu_msg(982);
 			unlock_task(session_lock,"remote_session");
 			goto err1;
 		}
-set_cpu_msg(982);
 		if ( get_type(ret) == XLT_ERROR ) {
 			switch ( ret->err.code ) {
 			case XLE_PROTO_ACCESS_STREAM:
@@ -1200,7 +1413,6 @@ printf("ERR1(%i)\n",get_tid());
 */
 					goto err1;
 				}
-set_cpu_msg(983);
 				read_unlock_agent(a);
 				goto retry;
 			}
@@ -1222,15 +1434,11 @@ err1:
 	goto err2;
 
 err:
-set_cpu_msg(985);
 	lock_task(session_lock);
 err2:
-set_cpu_msg(986);
 	if ( ses )
 		_free_session_lock(ses);
-set_cpu_msg(987);
 	unlock_task(session_lock,"close_session");
-set_cpu_msg(99);
 
 	if ( agent )
 		d_f_ree(agent);
@@ -1243,6 +1451,95 @@ set_cpu_msg(99);
 	return ret;
 }
 
+
+XL_SEXP *
+setup_session_proxy(int id,XL_FILE * f,int ln)
+{
+CON_SESSION * ses;
+XL_SEXP * ret;
+XL_SEXP * send_query,* sq;
+char buf[20];
+XL_SEXP * open_session;
+	lock_task(session_lock);
+	ses = _search_con_session(id);
+	if ( ses == 0 ) {
+		ret = get_error(
+			f,
+			ln,
+			XLE_PROTO_INV_PARAM,
+			l_string(std_cm,"RemoteSession"),
+			List(n_get_string("invalid session id"),
+				get_integer(id,0),
+				-1));
+		goto end;
+	}
+	if ( ses->flags & CSF_PROXY_INIT ) {
+		ret = 0;
+		goto end;
+	}
+	ses->flags |= CSF_PROXY_INIT;
+	if ( ses->pre_proxy )
+		sq = (*ses->pre_proxy)(ses);
+	else	sq = 0;
+	unlock_task(session_lock,"setup_session_proxy");
+
+	open_session = n_get_symbol("OpenSession");
+	set_attribute(open_session,
+		l_string(std_cm,"type"),
+		l_string(std_cm,"optimize"));
+	sprintf(buf,"rs%i",id);
+	send_query = List(
+		List(n_get_symbol("Define"),
+			n_get_symbol(buf),
+			List(open_session,
+				-1),
+			-1),
+		sq,
+		-1);
+
+	_remote_session(
+			gblisp_top_env0,
+			native_proxy_session,
+			0,
+			l_string(std_cm,"xlproxy"),
+			l_string(std_cm,"user"),
+			0,
+			send_query,
+			f,
+			ln,
+			0,
+			0,
+			ses->id);
+	return 0;
+end:
+	unlock_task(session_lock,"setup_session_proxy");
+	return ret;
+}
+
+
+NPROXY_DIRECT_SERVER *
+_search_direct_server(L_CHAR * server)
+{
+NPROXY_DIRECT_SERVER * ret;
+	for ( ret = np_server_list ; ret ; ret = ret->next ) {
+		if ( l_strcmp(server,ret->server) == 0 )
+			return ret;
+	}
+	return 0;
+}
+
+
+NPROXY_DIRECT_SERVER *
+search_direct_server(L_CHAR * server)
+{
+NPROXY_DIRECT_SERVER * ret;
+	lock_task(session_lock);
+	ret = _search_direct_server(server);
+	unlock_task(session_lock,"search_direct_server");
+	return ret;
+}
+
+
 XL_SEXP *
 remote_session(
 	XLISP_ENV * env,
@@ -1256,18 +1553,87 @@ remote_session(
 	int ln,
 	int session_lock_enable)
 {
+NPROXY_DIRECT_SERVER * is;
+XL_SEXP * send_query, * ret;
+char buf[20];
+	if ( native_proxy_session == 0 )
+		return _remote_session(
+				env,
+				id,
+				u,
+				a_agent,
+				a_login_mode,
+				a_center_cmd,
+				cmd,
+				f,
+				ln,
+				session_lock_enable,
+				0,0);
+	is = search_direct_server(u->server);
+	if ( is )
+		return _remote_session(
+				env,
+				id,
+				u,
+				a_agent,
+				a_login_mode,
+				a_center_cmd,
+				cmd,
+				f,
+				ln,
+				session_lock_enable,
+				0,0);
+
+
+	ret = setup_session_proxy(id,f,ln);
+	if ( ret )
+		return ret;
+
+	send_query = cmd;
+	if ( a_center_cmd )
+		send_query = cons(
+			List(n_get_symbol("Command"),
+				get_string(a_center_cmd),
+				-1),
+			send_query);
+	if ( a_login_mode )
+		send_query = cons(
+			List(n_get_symbol("LoginMode"),
+				get_string(a_login_mode),
+				-1),
+			send_query);
+	if ( a_agent )
+		send_query = cons(
+			List(n_get_symbol("Agent"),
+				get_string(a_agent),
+				-1),
+			send_query);
+	send_query = cons(
+		List(n_get_symbol("URL"),
+		get_string(get_url_str2(u)),
+		-1),
+		send_query);
+
+	sprintf(buf,"ps%i",id);
+
+	send_query = cons(
+		n_get_symbol(buf),
+		send_query);
+	send_query = cons(
+		n_get_symbol("RemoteSession"),
+		send_query);
 	return _remote_session(
 			env,
-			id,
-			u,
-			a_agent,
-			a_login_mode,
-			a_center_cmd,
-			cmd,
+			native_proxy_session,
+			&native_proxy_url,
+			l_string(std_cm,"xlproxy"),
+			l_string(std_cm,"user"),
+			0,
+			List(send_query,-1),
 			f,
 			ln,
 			session_lock_enable,
-			0);
+			0,id);
 }
 
 
@@ -1276,7 +1642,6 @@ remote_session_delay(XL_SEXP * s)
 {
 XL_RESULT * resp;
 XL_SEXP * ret;
-void gc_gb_sexp();
 int retry_cnt;
 extern SEM xli_lock;
 XL_SEXP * _wait_result_inp(XL_RESULT*);
@@ -1286,13 +1651,9 @@ CON_SESSION * ses;
 	retry_cnt = 3;
 retry:
 	gc_push(0,0,"remote_delay_func");
-set_t_msg(3000);
 	lock_task(xli_lock);
-set_t_msg(3001);
 	ret = _wait_result_inp(resp);
-set_t_msg(3002);
 	unlock_task(xli_lock,"remote_delay_func");
-set_t_msg(3003);
 	gc_pop(ret,gc_gb_sexp);
 
 	gc_push(0,0,"remote_delay_func");
@@ -1311,7 +1672,8 @@ set_t_msg(3003);
 				resp->s_center_cmd,
 				resp->s_cmd,
 				resp->s_file,
-				resp->s_line,0,1);
+				resp->s_line,0,1,
+				resp->s_proxy_ses);
 			if ( ret->h.type != XLT_DELAY )
 				goto end;
 			resp = (XL_RESULT*)ret->delay.d.func;
@@ -1376,3 +1738,121 @@ session_tick()
 	_connection_check();
 	unlock_task(session_lock,"session_tick");
 }
+
+int
+_set_native_proxy_direct_access(L_CHAR * server)
+{
+NPROXY_DIRECT_SERVER * d;
+	d = _search_direct_server(server);
+	if ( d )
+		return -1;
+	d = d_alloc(sizeof(*d),22);
+	d->server = ll_copy_str(server,22);
+	d->next = np_server_list;
+	np_server_list = d;
+	return 0;
+}
+
+
+int
+set_native_proxy_direct_access(L_CHAR * server)
+{
+int ret;
+	lock_task(session_lock);
+	ret = _set_native_proxy_direct_access(server);
+	unlock_task(session_lock,"set_native_proxy");
+	return ret;
+}
+
+
+int
+pre_proxy_func(
+	CON_SESSION * ses,
+	CON_AGENT * a,
+	int proxy_ses)
+{
+CON_SESSION * s;
+	lock_task(session_lock);
+	for ( s = session_list ; s ; s = s->next )
+		s->flags &= ~CSF_PROXY_INIT;
+	unlock_task(session_lock,"pre_proxy_func");
+
+	setup_session_proxy(proxy_ses,0,0);
+
+	return 0;
+}
+
+int
+set_native_proxy(URL * u)
+{
+CON_SESSION * ses;
+	if ( u->server == 0 ) {
+		close_session(native_proxy_session);
+		native_proxy_session = 0;
+		free_url(&native_proxy_url);
+		return 0;
+	}
+	if ( native_proxy_session == 0 )
+		native_proxy_session = open_session(SEST_OPTIMIZE);
+	free_url(&native_proxy_url);
+	copy_url(&native_proxy_url,u);
+
+	lock_task(session_lock);
+	ses = _search_con_session(native_proxy_session);
+	_set_pre_func(ses,
+		pre_proxy_func,
+		0,
+		0,
+		0,
+		0);
+	unlock_task(session_lock,"set_native_proxy");
+	return 0;
+}
+
+XL_SEXP *
+_get_native_proxy_server_list()
+{
+NPROXY_DIRECT_SERVER * s;
+XL_SEXP * ret;
+	ret = 0;
+	for ( s = server_list ; s ; s = s->next ) {
+		ret = cons(
+			List(n_get_symbol("server"),
+				n_get_string("NoProxy"),
+				get_string(s->server),
+				-1),
+			ret);
+	}
+	return ret;
+}
+
+
+XL_SEXP *
+_get_native_proxyinfo()
+{
+	if ( native_proxy_session ) {
+		return List(n_get_symbol("proxyinfo"),
+			n_get_string("NoProxy"),
+			-1);
+	}
+	else {
+		return append(List(n_get_symbol("proxyinfo"),
+			n_get_string("NativeProxy"),
+			get_string(native_proxy_url.server),
+			get_integer(native_proxy_url.port,0),
+			-1),
+			_get_native_proxy_server_list());
+	}
+}
+
+
+XL_SEXP *
+get_native_proxyinfo()
+{
+XL_SEXP *ret;
+	lock_task(session_lock);
+	ret = _get_native_proxyinfo();
+	unlock_task(session_lock,"get_native_proxyinfo");
+	return ret;
+}
+
